@@ -11,7 +11,7 @@ import matplotlib.pyplot as plt
 
 from ._core import VectorPlotter, variable_type, categorical_order
 from . import utils
-from .utils import _check_argument, adjust_legend_subtitles
+from .utils import _check_argument, adjust_legend_subtitles, _draw_figure
 from .palettes import color_palette, blend_palette
 from ._decorators import _deprecate_positional_args
 from ._docstrings import (
@@ -28,8 +28,45 @@ _param_docs = DocstringComponents.from_nested_components(
 )
 
 
-class Grid:
+class _BaseGrid:
     """Base class for grids of subplots."""
+
+    def set(self, **kwargs):
+        """Set attributes on each subplot Axes."""
+        for ax in self.axes.flat:
+            if ax is not None:  # Handle removed axes
+                ax.set(**kwargs)
+        return self
+
+    @property
+    def fig(self):
+        """DEPRECATED: prefer the `figure` property."""
+        # Grid.figure is preferred because it matches the Axes attribute name.
+        # But as the maintanace burden on having this property is minimal,
+        # let's be slow about formally deprecating it. For now just note its deprecation
+        # in the docstring; add a warning in version 0.13, and eventually remove it.
+        return self._figure
+
+    @property
+    def figure(self):
+        """Access the :class:`matplotlib.figure.Figure` object underlying the grid."""
+        return self._figure
+
+    def savefig(self, *args, **kwargs):
+        """
+        Save an image of the plot.
+
+        This wraps :meth:`matplotlib.figure.Figure.savefig`, using bbox_inches="tight"
+        by default. Parameters are passed through to the matplotlib function.
+
+        """
+        kwargs = kwargs.copy()
+        kwargs.setdefault("bbox_inches", "tight")
+        self.figure.savefig(*args, **kwargs)
+
+
+class Grid(_BaseGrid):
+    """A grid that can have multiple subplots and an external legend."""
     _margin_titles = False
     _legend_out = True
 
@@ -42,26 +79,13 @@ class Grid:
         # don't add proxy artists onto the Axes. We need an overall cleaner approach.
         self._extract_legend_handles = False
 
-    def set(self, **kwargs):
-        """Set attributes on each subplot Axes."""
-        for ax in self.axes.flat:
-            if ax is not None:  # Handle removed axes
-                ax.set(**kwargs)
-        return self
-
-    def savefig(self, *args, **kwargs):
-        """Save the figure."""
-        kwargs = kwargs.copy()
-        kwargs.setdefault("bbox_inches", "tight")
-        self.fig.savefig(*args, **kwargs)
-
     def tight_layout(self, *args, **kwargs):
         """Call fig.tight_layout within rect that exclude the legend."""
         kwargs = kwargs.copy()
         kwargs.setdefault("rect", self._tight_layout_rect)
         if self._tight_layout_pad is not None:
             kwargs.setdefault("pad", self._tight_layout_pad)
-        self.fig.tight_layout(*args, **kwargs)
+        self._figure.tight_layout(*args, **kwargs)
 
     def add_legend(self, legend_data=None, title=None, label_order=None,
                    adjust_subtitles=False, **kwargs):
@@ -129,7 +153,7 @@ class Grid:
             kwargs.setdefault("loc", "center right")
 
             # Draw a full-figure legend outside the grid
-            figlegend = self.fig.legend(handles, labels, **kwargs)
+            figlegend = self._figure.legend(handles, labels, **kwargs)
 
             self._legend = figlegend
             figlegend.set_title(title, prop={"size": title_size})
@@ -138,27 +162,25 @@ class Grid:
                 adjust_legend_subtitles(figlegend)
 
             # Draw the plot to set the bounding boxes correctly
-            if hasattr(self.fig.canvas, "get_renderer"):
-                self.fig.draw(self.fig.canvas.get_renderer())
+            _draw_figure(self._figure)
 
             # Calculate and set the new width of the figure so the legend fits
-            legend_width = figlegend.get_window_extent().width / self.fig.dpi
-            fig_width, fig_height = self.fig.get_size_inches()
-            self.fig.set_size_inches(fig_width + legend_width, fig_height)
+            legend_width = figlegend.get_window_extent().width / self._figure.dpi
+            fig_width, fig_height = self._figure.get_size_inches()
+            self._figure.set_size_inches(fig_width + legend_width, fig_height)
 
             # Draw the plot again to get the new transformations
-            if hasattr(self.fig.canvas, "get_renderer"):
-                self.fig.draw(self.fig.canvas.get_renderer())
+            _draw_figure(self._figure)
 
             # Now calculate how much space we need on the right side
-            legend_width = figlegend.get_window_extent().width / self.fig.dpi
+            legend_width = figlegend.get_window_extent().width / self._figure.dpi
             space_needed = legend_width / (fig_width + legend_width)
             margin = .04 if self._margin_titles else .01
             self._space_needed = margin + space_needed
             right = 1 - self._space_needed
 
             # Place the subplot axes to give space for the legend
-            self.fig.subplots_adjust(right=right)
+            self._figure.subplots_adjust(right=right)
             self._tight_layout_rect[2] = right
 
         else:
@@ -175,16 +197,12 @@ class Grid:
 
         return self
 
-    def _clean_axis(self, ax):
-        """Turn off axis labels and legend."""
-        ax.set_xlabel("")
-        ax.set_ylabel("")
-        ax.legend_ = None
-        return self
-
     def _update_legend_data(self, ax):
         """Extract the legend data from an axes object and save it."""
         data = {}
+
+        # Get data directly from the legend, which is necessary
+        # for newer functions that don't add labeled proxy artists
         if ax.legend_ is not None and self._extract_legend_handles:
             handles = ax.legend_.legendHandles
             labels = [t.get_text() for t in ax.legend_.texts]
@@ -195,6 +213,9 @@ class Grid:
 
         self._legend_data.update(data)
 
+        # Now clear the legend
+        ax.legend_ = None
+
     def _get_palette(self, data, hue, hue_order, palette):
         """Get a list of colors for the hue variable."""
         if hue is None:
@@ -382,14 +403,19 @@ class FacetGrid(Grid):
             subplot_kws["ylim"] = ylim
 
         # --- Initialize the subplot grid
+
+        # Disable autolayout so legend_out works properly
+        with mpl.rc_context({"figure.autolayout": False}):
+            fig = plt.figure(figsize=figsize)
+
         if col_wrap is None:
 
-            kwargs = dict(figsize=figsize, squeeze=False,
+            kwargs = dict(squeeze=False,
                           sharex=sharex, sharey=sharey,
                           subplot_kw=subplot_kws,
                           gridspec_kw=gridspec_kws)
 
-            fig, axes = plt.subplots(nrow, ncol, **kwargs)
+            axes = fig.subplots(nrow, ncol, **kwargs)
 
             if col is None and row is None:
                 axes_dict = {}
@@ -408,7 +434,6 @@ class FacetGrid(Grid):
                 warnings.warn("`gridspec_kws` ignored when using `col_wrap`")
 
             n_axes = len(col_names)
-            fig = plt.figure(figsize=figsize)
             axes = np.empty(n_axes, object)
             axes[0] = fig.add_subplot(nrow, ncol, 1, **subplot_kws)
             if sharex:
@@ -424,7 +449,7 @@ class FacetGrid(Grid):
 
         # Attributes that are part of the public API but accessed through
         # a  property so that Sphinx adds them to the auto class doc
-        self._fig = fig
+        self._figure = fig
         self._axes = axes
         self._axes_dict = axes_dict
         self._legend = None
@@ -466,12 +491,14 @@ class FacetGrid(Grid):
                 for label in ax.get_xticklabels():
                     label.set_visible(False)
                 ax.xaxis.offsetText.set_visible(False)
+                ax.xaxis.label.set_visible(False)
 
         if sharey in [True, 'row']:
             for ax in self._not_left_axes:
                 for label in ax.get_yticklabels():
                     label.set_visible(False)
                 ax.yaxis.offsetText.set_visible(False)
+                ax.yaxis.label.set_visible(False)
 
     __init__.__doc__ = dedent("""\
         Initialize the matplotlib figure and FacetGrid object.
@@ -540,7 +567,7 @@ class FacetGrid(Grid):
         gridspec_kws : dict
             Dictionary of keyword arguments passed to
             :class:`matplotlib.gridspec.GridSpec`
-            (via :func:`matplotlib.pyplot.subplots`).
+            (via :meth:`matplotlib.figure.Figure.subplots`).
             Ignored if ``col_wrap`` is not ``None``.
 
         See Also
@@ -749,8 +776,12 @@ class FacetGrid(Grid):
             # Draw the plot
             self._facet_plot(func, ax, args, kwargs)
 
-        # Finalize the annotations and layout
-        self._finalize_grid(args[:2])
+        # For axis labels, prefer to use positional args for backcompat
+        # but also extract the x/y kwargs and use if no corresponding arg
+        axis_labels = [kwargs.get("x", None), kwargs.get("y", None)]
+        for i, val in enumerate(args[:2]):
+            axis_labels[i] = val
+        self._finalize_grid(axis_labels)
 
         return self
 
@@ -776,7 +807,6 @@ class FacetGrid(Grid):
 
         # Sort out the supporting information
         self._update_legend_data(ax)
-        self._clean_axis(ax)
 
     def _finalize_grid(self, axlabels):
         """Finalize the annotations and layout."""
@@ -800,7 +830,7 @@ class FacetGrid(Grid):
 
     def despine(self, **kwargs):
         """Remove axis spines from the facets."""
-        utils.despine(self.fig, **kwargs)
+        utils.despine(self._figure, **kwargs)
         return self
 
     def set_axis_labels(self, x_var=None, y_var=None, clear_inner=True, **kwargs):
@@ -956,12 +986,39 @@ class FacetGrid(Grid):
                 self.axes.flat[i].set_title(title, **kwargs)
         return self
 
-    # ------ Properties that are part of the public API and documented by Sphinx
+    def refline(self, *, x=None, y=None, color='.5', linestyle='--', **line_kws):
+        """Add a reference line(s) to each facet.
 
-    @property
-    def fig(self):
-        """The :class:`matplotlib.figure.Figure` with the plot."""
-        return self._fig
+        Parameters
+        ----------
+        x, y : numeric
+            Value(s) to draw the line(s) at.
+        color : :mod:`matplotlib color <matplotlib.colors>`
+            Specifies the color of the reference line(s). Pass ``color=None`` to
+            use ``hue`` mapping.
+        linestyle : str
+            Specifies the style of the reference line(s).
+        line_kws : key, value mappings
+            Other keyword arguments are passed to :meth:`matplotlib.axes.Axes.axvline`
+            when ``x`` is not None and :meth:`matplotlib.axes.Axes.axhline` when ``y``
+            is not None.
+
+        Returns
+        -------
+        :class:`FacetGrid` instance
+            Returns ``self`` for easy method chaining.
+
+        """
+        line_kws['color'] = color
+        line_kws['linestyle'] = linestyle
+
+        if x is not None:
+            self.map(plt.axvline, x=x, **line_kws)
+
+        if y is not None:
+            self.map(plt.axhline, y=y, **line_kws)
+
+    # ------ Properties that are part of the public API and documented by Sphinx
 
     @property
     def axes(self):
@@ -1180,16 +1237,19 @@ class PairGrid(Grid):
         # Create the figure and the array of subplots
         figsize = len(x_vars) * height * aspect, len(y_vars) * height
 
-        fig, axes = plt.subplots(len(y_vars), len(x_vars),
-                                 figsize=figsize,
-                                 sharex="col", sharey="row",
-                                 squeeze=False)
+        # Disable autolayout so legend_out works
+        with mpl.rc_context({"figure.autolayout": False}):
+            fig = plt.figure(figsize=figsize)
+
+        axes = fig.subplots(len(y_vars), len(x_vars),
+                            sharex="col", sharey="row",
+                            squeeze=False)
 
         # Possibly remove upper axes to make a corner grid
         # Note: setting up the axes is usually the most time-intensive part
         # of using the PairGrid. We are foregoing the speed improvement that
         # we would get by just not setting up the hidden axes so that we can
-        # avoid implementing plt.subplots ourselves. But worth thinking about.
+        # avoid implementing fig.subplots ourselves. But worth thinking about.
         self._corner = corner
         if corner:
             hide_indices = np.triu_indices_from(axes, 1)
@@ -1197,7 +1257,7 @@ class PairGrid(Grid):
                 axes[i, j].remove()
                 axes[i, j] = None
 
-        self.fig = fig
+        self._figure = fig
         self.axes = axes
         self.data = data
 
@@ -1242,6 +1302,22 @@ class PairGrid(Grid):
         self._legend_data = {}
 
         # Make the plot look nice
+        for ax in axes[:-1, :].flat:
+            if ax is None:
+                continue
+            for label in ax.get_xticklabels():
+                label.set_visible(False)
+            ax.xaxis.offsetText.set_visible(False)
+            ax.xaxis.label.set_visible(False)
+
+        for ax in axes[:, 1:].flat:
+            if ax is None:
+                continue
+            for label in ax.get_yticklabels():
+                label.set_visible(False)
+            ax.yaxis.offsetText.set_visible(False)
+            ax.yaxis.label.set_visible(False)
+
         self._tight_layout_rect = [.01, .01, .99, .99]
         self._tight_layout_pad = layout_pad
         self._despine = despine
@@ -1400,7 +1476,7 @@ class PairGrid(Grid):
             plot_kwargs.setdefault("hue_order", self._hue_order)
             plot_kwargs.setdefault("palette", self._orig_palette)
             func(x=vector, **plot_kwargs)
-            self._clean_axis(ax)
+            ax.legend_ = None
 
         self._add_axis_labels()
         return self
@@ -1440,8 +1516,6 @@ class PairGrid(Grid):
                 else:
                     func(data_k, label=label_k, color=color, **plot_kwargs)
 
-            self._clean_axis(ax)
-
         self._add_axis_labels()
 
         return self
@@ -1505,7 +1579,6 @@ class PairGrid(Grid):
         func(x=x, y=y, **kwargs)
 
         self._update_legend_data(ax)
-        self._clean_axis(ax)
 
     def _plot_bivariate_iter_hue(self, x_var, y_var, ax, func, **kwargs):
         """Draw a bivariate plot while iterating over hue subsets."""
@@ -1550,7 +1623,6 @@ class PairGrid(Grid):
                 func(x, y, **kws)
 
         self._update_legend_data(ax)
-        self._clean_axis(ax)
 
     def _add_axis_labels(self):
         """Add labels to the left and bottom Axes."""
@@ -1570,7 +1642,7 @@ class PairGrid(Grid):
         return numeric_cols
 
 
-class JointGrid(object):
+class JointGrid(_BaseGrid):
     """Grid for drawing a bivariate plot with marginal univariate plots.
 
     Many plots can be drawn by using the figure-level interface :func:`jointplot`.
@@ -1602,7 +1674,7 @@ class JointGrid(object):
         ax_marg_x = f.add_subplot(gs[0, :-1], sharex=ax_joint)
         ax_marg_y = f.add_subplot(gs[1:, -1], sharey=ax_joint)
 
-        self.fig = f
+        self._figure = f
         self.ax_joint = ax_joint
         self.ax_marg_x = ax_marg_x
         self.ax_marg_y = ax_marg_y
@@ -1684,7 +1756,7 @@ class JointGrid(object):
 
         Parameters
         ----------
-        joint_func, marginal_func: callables
+        joint_func, marginal_func : callables
             Functions to draw the bivariate and univariate plots. See methods
             referenced above for information about the required characteristics
             of these functions.
@@ -1798,6 +1870,50 @@ class JointGrid(object):
 
         return self
 
+    def refline(
+        self, *, x=None, y=None, joint=True, marginal=True,
+        color='.5', linestyle='--', **line_kws
+    ):
+        """Add a reference line(s) to joint and/or marginal axes.
+
+        Parameters
+        ----------
+        x, y : numeric
+            Value(s) to draw the line(s) at.
+        joint, marginal : bools
+            Whether to add the reference line(s) to the joint/marginal axes.
+        color : :mod:`matplotlib color <matplotlib.colors>`
+            Specifies the color of the reference line(s).
+        linestyle : str
+            Specifies the style of the reference line(s).
+        line_kws : key, value mappings
+            Other keyword arguments are passed to :meth:`matplotlib.axes.Axes.axvline`
+            when ``x`` is not None and :meth:`matplotlib.axes.Axes.axhline` when ``y``
+            is not None.
+
+        Returns
+        -------
+        :class:`JointGrid` instance
+            Returns ``self`` for easy method chaining.
+
+        """
+        line_kws['color'] = color
+        line_kws['linestyle'] = linestyle
+
+        if x is not None:
+            if joint:
+                self.ax_joint.axvline(x, **line_kws)
+            if marginal:
+                self.ax_marg_x.axvline(x, **line_kws)
+
+        if y is not None:
+            if joint:
+                self.ax_joint.axhline(y, **line_kws)
+            if marginal:
+                self.ax_marg_y.axhline(y, **line_kws)
+
+        return self
+
     def set_axis_labels(self, xlabel="", ylabel="", **kwargs):
         """Set axis labels on the bivariate axes.
 
@@ -1821,15 +1937,6 @@ class JointGrid(object):
         self.ax_joint.set_ylabel(ylabel, **kwargs)
         return self
 
-    def savefig(self, *args, **kwargs):
-        """Save the figure using a "tight" bounding box by default.
-
-        Wraps :meth:`matplotlib.figure.Figure.savefig`.
-
-        """
-        kwargs.setdefault("bbox_inches", "tight")
-        self.fig.savefig(*args, **kwargs)
-
 
 JointGrid.__init__.__doc__ = """\
 Set up the grid of subplots and store data internally for easy plotting.
@@ -2019,6 +2126,7 @@ def pairplot(
         grid.map_diag(histplot, **diag_kws)
     elif diag_kind == "kde":
         diag_kws.setdefault("fill", True)
+        diag_kws.setdefault("warn_singular", False)
         grid.map_diag(kdeplot, **diag_kws)
 
     # Maybe plot on the off-diagonals
@@ -2035,6 +2143,7 @@ def pairplot(
         plotter(regplot, **plot_kws)
     elif kind == "kde":
         from .distributions import kdeplot  # Avoid circular import
+        plot_kws.setdefault("warn_singular", False)
         plotter(kdeplot, **plot_kws)
     elif kind == "hist":
         from .distributions import histplot  # Avoid circular import
@@ -2138,6 +2247,7 @@ def jointplot(
             marg_func = histplot
         else:
             marg_func = kdeplot
+            marginal_kws.setdefault("warn_singular", False)
             marginal_kws.setdefault("fill", True)
 
         marginal_kws.setdefault("color", color)
@@ -2170,6 +2280,7 @@ def jointplot(
     elif kind.startswith("kde"):
 
         joint_kws.setdefault("color", color)
+        joint_kws.setdefault("warn_singular", False)
         grid.plot_joint(kdeplot, **joint_kws)
 
         marginal_kws.setdefault("color", color)
