@@ -1,5 +1,4 @@
 """Plotting functions for linear models (broadly construed)."""
-from __future__ import division
 import copy
 from textwrap import dedent
 import warnings
@@ -19,6 +18,7 @@ except ImportError:
 from . import utils
 from . import algorithms as algo
 from .axisgrid import FacetGrid, _facet_docs
+from ._decorators import _deprecate_positional_args
 
 
 __all__ = ["lmplot", "regplot", "residplot"]
@@ -48,7 +48,7 @@ class _LinearPlotter(object):
                 vector = np.asarray(val)
             else:
                 vector = val
-            if vector is not None:
+            if vector is not None and vector.shape != (1,):
                 vector = np.squeeze(vector)
             if np.ndim(vector) > 1:
                 err = "regplot inputs must be 1d"
@@ -127,8 +127,13 @@ class _RegressionPlotter(_LinearPlotter):
         else:
             self.x_discrete = self.x
 
+        # Disable regression in case of singleton inputs
+        if len(self.x) <= 1:
+            self.fit_reg = False
+
         # Save the range of the x variable for the grid later
-        self.x_range = self.x.min(), self.x.max()
+        if self.fit_reg:
+            self.x_range = self.x.min(), self.x.max()
 
     @property
     def scatter_data(self):
@@ -316,7 +321,7 @@ class _RegressionPlotter(_LinearPlotter):
         x = self.x
         if np.isscalar(bins):
             percentiles = np.linspace(0, 100, bins + 2)[1:-1]
-            bins = np.c_[utils.percentiles(x, percentiles)]
+            bins = np.c_[np.percentile(x, percentiles)]
         else:
             bins = np.c_[np.ravel(bins)]
 
@@ -415,11 +420,8 @@ class _RegressionPlotter(_LinearPlotter):
 
         # Draw the regression line and confidence interval
         line, = ax.plot(grid, yhat, **kws)
-        try:
+        if not self.truncate:
             line.sticky_edges.x[:] = edges  # Prevent mpl from adding margin
-        except AttributeError:
-            msg = "Cannot set sticky_edges; requires newer matplotlib."
-            warnings.warn(msg, UserWarning)
         if err_bands is not None:
             ax.fill_between(grid, *err_bands, facecolor=fill_color, alpha=.15)
 
@@ -536,9 +538,8 @@ _regression_docs = dict(
     """),
     truncate=dedent("""\
     truncate : bool, optional
-        By default, the regression line is drawn to fill the x axis limits
-        after the scatterplot is drawn. If ``truncate`` is ``True``, it will
-        instead by bounded by the data limits.\
+        If ``True``, the regression line is bounded by the data limits. If
+        ``False``, it extends to the ``x`` axis limits.
     """),
     xy_jitter=dedent("""\
     {x,y}_jitter : floats, optional
@@ -552,19 +553,25 @@ _regression_docs = dict(
         Additional keyword arguments to pass to ``plt.scatter`` and
         ``plt.plot``.\
     """),
-    )
+)
 _regression_docs.update(_facet_docs)
 
 
-def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
-           col_wrap=None, height=5, aspect=1, markers="o", sharex=True,
-           sharey=True, hue_order=None, col_order=None, row_order=None,
-           legend=True, legend_out=True, x_estimator=None, x_bins=None,
-           x_ci="ci", scatter=True, fit_reg=True, ci=95, n_boot=1000,
-           units=None, seed=None, order=1, logistic=False, lowess=False,
-           robust=False, logx=False, x_partial=None, y_partial=None,
-           truncate=True, x_jitter=None, y_jitter=None, scatter_kws=None,
-           line_kws=None, size=None):
+@_deprecate_positional_args
+def lmplot(
+    *,
+    x=None, y=None,
+    data=None,
+    hue=None, col=None, row=None,  # TODO move before data once * is enforced
+    palette=None, col_wrap=None, height=5, aspect=1, markers="o",
+    sharex=None, sharey=None, hue_order=None, col_order=None, row_order=None,
+    legend=True, legend_out=None, x_estimator=None, x_bins=None,
+    x_ci="ci", scatter=True, fit_reg=True, ci=95, n_boot=1000,
+    units=None, seed=None, order=1, logistic=False, lowess=False,
+    robust=False, logx=False, x_partial=None, y_partial=None,
+    truncate=True, x_jitter=None, y_jitter=None, scatter_kws=None,
+    line_kws=None, facet_kws=None, size=None,
+):
 
     # Handle deprecations
     if size is not None:
@@ -573,17 +580,38 @@ def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
                "please update your code.")
         warnings.warn(msg, UserWarning)
 
+    if facet_kws is None:
+        facet_kws = {}
+
+    def facet_kw_deprecation(key, val):
+        msg = (
+            f"{key} is deprecated from the `lmplot` function signature. "
+            "Please update your code to pass it using `facet_kws`."
+        )
+        if val is not None:
+            warnings.warn(msg, UserWarning)
+            facet_kws[key] = val
+
+    facet_kw_deprecation("sharex", sharex)
+    facet_kw_deprecation("sharey", sharey)
+    facet_kw_deprecation("legend_out", legend_out)
+
+    if data is None:
+        raise TypeError("Missing required keyword argument `data`.")
+
     # Reduce the dataframe to only needed columns
     need_cols = [x, y, hue, col, row, units, x_partial, y_partial]
     cols = np.unique([a for a in need_cols if a is not None]).tolist()
     data = data[cols]
 
     # Initialize the grid
-    facets = FacetGrid(data, row, col, hue, palette=palette,
-                       row_order=row_order, col_order=col_order,
-                       hue_order=hue_order, height=height, aspect=aspect,
-                       col_wrap=col_wrap, sharex=sharex, sharey=sharey,
-                       legend_out=legend_out)
+    facets = FacetGrid(
+        data, row=row, col=col, hue=hue,
+        palette=palette,
+        row_order=row_order, col_order=col_order, hue_order=hue_order,
+        height=height, aspect=aspect, col_wrap=col_wrap,
+        **facet_kws,
+    )
 
     # Add the markers here as FacetGrid has figured out how many levels of the
     # hue variable are needed and we don't want to duplicate that process
@@ -598,12 +626,12 @@ def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
                           "for each level of the hue variable"))
     facets.hue_kws = {"marker": markers}
 
-    # Hack to set the x limits properly, which needs to happen here
-    # because the extent of the regression estimate is determined
-    # by the limits of the plot
-    if sharex:
-        for ax in facets.axes.flat:
-            ax.scatter(data[x], np.ones(len(data)) * data[y].mean()).remove()
+    def update_datalim(data, x, y, ax, **kws):
+        xys = np.asarray(data[[x, y]]).astype(float)
+        ax.update_datalim(xys, updatey=False)
+        ax.autoscale_view(scaley=False)
+
+    facets.map_dataframe(update_datalim, x=x, y=y)
 
     # Draw the regression plot on each facet
     regplot_kws = dict(
@@ -613,8 +641,9 @@ def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
         robust=robust, logx=logx, x_partial=x_partial, y_partial=y_partial,
         truncate=truncate, x_jitter=x_jitter, y_jitter=y_jitter,
         scatter_kws=scatter_kws, line_kws=line_kws,
-        )
-    facets.map_dataframe(regplot, x, y, **regplot_kws)
+    )
+    facets.map_dataframe(regplot, x=x, y=y, **regplot_kws)
+    facets.set_axis_labels(x, y)
 
     # Add a legend
     if legend and (hue is not None) and (hue not in [col, row]):
@@ -658,6 +687,10 @@ lmplot.__doc__ = dedent("""\
         Markers for the scatterplot. If a list, each marker in the list will be
         used for each level of the ``hue`` variable.
     {share_xy}
+
+        .. deprecated:: 0.12.0
+            Pass using the `facet_kws` dictionary.
+
     {{hue,col,row}}_order : lists, optional
         Order for the levels of the faceting variables. By default, this will
         be the order that the levels appear in ``data`` or, if the variables
@@ -665,6 +698,10 @@ lmplot.__doc__ = dedent("""\
     legend : bool, optional
         If ``True`` and there is a ``hue`` variable, add a legend.
     {legend_out}
+
+        .. deprecated:: 0.12.0
+            Pass using the `facet_kws` dictionary.
+
     {x_estimator}
     {x_bins}
     {x_ci}
@@ -683,6 +720,8 @@ lmplot.__doc__ = dedent("""\
     {truncate}
     {xy_jitter}
     {scatter_line_kws}
+    facet_kws : dict
+        Dictionary of keyword arguments for :class:`FacetGrid`.
 
     See Also
     --------
@@ -710,7 +749,7 @@ lmplot.__doc__ = dedent("""\
     .. plot::
         :context: close-figs
 
-        >>> import seaborn as sns; sns.set(color_codes=True)
+        >>> import seaborn as sns; sns.set_theme(color_codes=True)
         >>> tips = sns.load_dataset("tips")
         >>> g = sns.lmplot(x="total_bill", y="tip", data=tips)
 
@@ -795,13 +834,19 @@ lmplot.__doc__ = dedent("""\
     """).format(**_regression_docs)
 
 
-def regplot(x, y, data=None, x_estimator=None, x_bins=None, x_ci="ci",
-            scatter=True, fit_reg=True, ci=95, n_boot=1000, units=None,
-            seed=None, order=1, logistic=False, lowess=False, robust=False,
-            logx=False, x_partial=None, y_partial=None,
-            truncate=True, dropna=True, x_jitter=None, y_jitter=None,
-            label=None, color=None, marker="o",
-            scatter_kws=None, line_kws=None, ax=None):
+@_deprecate_positional_args
+def regplot(
+    *,
+    x=None, y=None,
+    data=None,
+    x_estimator=None, x_bins=None, x_ci="ci",
+    scatter=True, fit_reg=True, ci=95, n_boot=1000, units=None,
+    seed=None, order=1, logistic=False, lowess=False, robust=False,
+    logx=False, x_partial=None, y_partial=None,
+    truncate=True, dropna=True, x_jitter=None, y_jitter=None,
+    label=None, color=None, marker="o",
+    scatter_kws=None, line_kws=None, ax=None
+):
 
     plotter = _RegressionPlotter(x, y, data, x_estimator, x_bins, x_ci,
                                  scatter, fit_reg, ci, n_boot, units, seed,
@@ -894,7 +939,7 @@ regplot.__doc__ = dedent("""\
     .. plot::
         :context: close-figs
 
-        >>> import seaborn as sns; sns.set(color_codes=True)
+        >>> import seaborn as sns; sns.set_theme(color_codes=True)
         >>> tips = sns.load_dataset("tips")
         >>> ax = sns.regplot(x="total_bill", y="tip", data=tips)
 
@@ -988,9 +1033,15 @@ regplot.__doc__ = dedent("""\
     """).format(**_regression_docs)
 
 
-def residplot(x, y, data=None, lowess=False, x_partial=None, y_partial=None,
-              order=1, robust=False, dropna=True, label=None, color=None,
-              scatter_kws=None, line_kws=None, ax=None):
+@_deprecate_positional_args
+def residplot(
+    *,
+    x=None, y=None,
+    data=None,
+    lowess=False, x_partial=None, y_partial=None,
+    order=1, robust=False, dropna=True, label=None, color=None,
+    scatter_kws=None, line_kws=None, ax=None
+):
     """Plot the residuals of a linear regression.
 
     This function will regress y on x (possibly as a robust or polynomial
