@@ -707,6 +707,32 @@ class TestRelationalPlotter(Helpers):
         for line, color in zip(lines, palette):
             assert line.get_color() == color
 
+    def test_relplot_data(self, long_df):
+
+        g = relplot(
+            data=long_df.to_dict(orient="list"),
+            x="x",
+            y=long_df["y"].rename("y_var"),
+            hue=np.asarray(long_df["a"]),
+            col="c",
+        )
+        expected_cols = set(long_df.columns.tolist() + ["_hue_", "y_var"])
+        assert set(g.data.columns) == expected_cols
+        assert_array_equal(g.data["y_var"], long_df["y"])
+        assert_array_equal(g.data["_hue_"], long_df["a"])
+
+    def test_facet_variable_collision(self, long_df):
+
+        # https://github.com/mwaskom/seaborn/issues/2488
+        col_data = long_df["c"]
+        long_df = long_df.assign(size=col_data)
+
+        g = relplot(
+            data=long_df,
+            x="x", y="y", col="size",
+        )
+        assert g.axes.shape == (1, len(col_data.unique()))
+
     def test_ax_kwarg_removal(self, long_df):
 
         f, ax = plt.subplots()
@@ -1688,6 +1714,40 @@ class TestScatterPlotter(Helpers):
         scatterplot(data=long_df, x="x", y="y", linewidth=lw)
         assert ax.collections[0].get_linewidths().item() == lw
 
+    def test_size_norm_extrapolation(self):
+
+        # https://github.com/mwaskom/seaborn/issues/2539
+        x = np.arange(0, 20, 2)
+        f, axs = plt.subplots(1, 2, sharex=True, sharey=True)
+
+        slc = 5
+        kws = dict(sizes=(50, 200), size_norm=(0, x.max()), legend="brief")
+
+        scatterplot(x=x, y=x, size=x, ax=axs[0], **kws)
+        scatterplot(x=x[:slc], y=x[:slc], size=x[:slc], ax=axs[1], **kws)
+
+        assert np.allclose(
+            axs[0].collections[0].get_sizes()[:slc],
+            axs[1].collections[0].get_sizes()
+        )
+
+        legends = [ax.legend_ for ax in axs]
+        legend_data = [
+            {
+                label.get_text(): handle.get_sizes().item()
+                for label, handle in zip(legend.get_texts(), legend.legendHandles)
+            } for legend in legends
+        ]
+
+        for key in set(legend_data[0]) & set(legend_data[1]):
+            if key == "y":
+                # At some point (circa 3.0) matplotlib auto-added pandas series
+                # with a valid name into the legend, which messes up this test.
+                # I can't track down when that was added (or removed), so let's
+                # just anticipate and ignore it here.
+                continue
+            assert legend_data[0][key] == legend_data[1][key]
+
     def test_datetime_scale(self, long_df):
 
         ax = scatterplot(data=long_df, x="t", y="y")
@@ -1695,6 +1755,12 @@ class TestScatterPlotter(Helpers):
         # https://github.com/matplotlib/matplotlib/issues/17586
         ax.get_xlim()[0] > ax.xaxis.convert_units(np.datetime64("2002-01-01"))
 
+    def test_unfilled_marker_edgecolor_warning(self, long_df):  # GH2636
+
+        with pytest.warns(None) as record:
+            scatterplot(data=long_df, x="x", y="y", marker="+")
+        assert not record
+
     def test_scatterplot_vs_relplot(self, long_df, long_semantics):
 
         ax = scatterplot(data=long_df, **long_semantics)
